from typing import Dict, Iterable, Any
import time, json, os

import bottle

from logger import Logger
from servicestub import ServiceStub
from singleton import Singleton


class Oplati(metaclass=Singleton):
    log: Logger
    cases: Dict[str, Iterable[Dict[str, Any]]]

    def __init__(self):
        self.log = Logger()
        self.cases = {}

    def registerRoutes(self, servicestub: ServiceStub):
        self.log.info('Регистрация сервиса')
        self.__updateCases()

        servicestub.route('/tokens', method='GET', callback=self.getToken)
        servicestub.route('/pos/demandPayment', method='POST', callback=self.demandPayment)
        servicestub.route('/pos/consumerStatus', method='GET', callback=self.consumerStatus)
        servicestub.route('/pos/async/payments', method='POST', callback=self.asyncPayments)
        servicestub.route('/pos/payments/<orderNumber>', method='DELETE', callback=self.deleteByOrderNumber)

    def getToken(self):
        self.log.info("Получение токена")
        methodPath = '/tokens'

        return self.__getResult(methodPath)

    def demandPayment(self):
        self.log.info("Требование оплаты")
        methodPath = '/pos/demandPayment'

        return self.__getResult(methodPath)

    def consumerStatus(self):
        self.log.info("Получение сведений о готовности клиента")
        methodPath = '/pos/consumerStatus'

        return self.__getResult(methodPath)

    def asyncPayments(self):
        self.log.info("Отправить запрос на осуществление платежа (с ожиданием)")
        methodPath = '/pos/async/payments'

        return self.__getResult(methodPath)

    def deleteByOrderNumber(self, orderNumber=''):
        self.log.info("Техническая отмена последней операции: '%s'" % orderNumber)
        methodPath = '/pos/payments/' + orderNumber

        if methodPath not in self.cases:
            methodPath = '/pos/payments/<orderNumber>'

        return self.__getResult(methodPath)

    def __getResult(self, methodPath: str):
        if methodPath not in self.cases:
            self.log.warn('Путь "%s" не существует' % methodPath)
            raise bottle.HTTPError(400, 'Путь "%s" не существует' % methodPath)

        if not len(self.cases[methodPath]):
            self.log.warn('В кейсах не осталось значений')
            return None

        time.sleep(self.cases[methodPath][0].get('timeout', 0))

        body = json.dumps(self.cases[methodPath][0].get('response'))
        response = bottle.HTTPResponse(status=self.cases[methodPath][0].get('status'), body=body)
        response.add_header("Content-Type", "application/json")

        headers = self.cases[methodPath][0].get('headers')
        if headers is not None:
            for header in headers.keys():
                response.add_header(header, headers[header])

        # если в кейсе нет счётчика, то возвращаем str(response)
        if 'count' not in self.cases[methodPath][0]:
            return response

        self.cases[methodPath][0]['count'] -= 1

        if self.cases[methodPath][0]['count'] == 0:
            del self.cases[methodPath][0]

        return response
    
    def __updateCases(self):
        try:
            with open(os.path.join(os.path.dirname(os.path.abspath(__file__)), 'oplati.json'), 'r') as cases:
                self.cases = json.load(cases)
        except Exception as e:
            self.log.error(str(e))


instance = Oplati()
